class Reactive(dict):
    def __init__(self, target, func):
        super().__init__(target)
        self.target = target
        self.func = func
        self.react()

    def react(self):
        for key in self.target:
            if type(self.target[key]) == dict:
                self.target[key] = Reactive(self.target[key], self.func)

    def __getitem__(self, key):
        return self.target[key]

    def __setitem__(self, key, value):
        if type(value) == dict:
            self.target[key] = Reactive(value, self.func)
        else:
            self.target[key] = value
        self.func()

    def __str__(self):
        return str(self.target)

    def read(self):
        return self.__str__()



import os
import json
from .reactive import Reactive


class Storage(object):
    def __init__(self, state={}, filename="state.json", filepath=os.getcwd(), force=False):
        self.path = filepath
        self.file = f'{filepath}/{filename}'

        if not self.load() or force:
            self.state = Reactive(state, self.save)
            self.save()

    def load(self):
        if os.path.exists(self.file):
            with open(self.file, "r") as storage:
                data = json.load(storage)
                if data == "":
                    self.state = Reactive({}, self.save)
                else:
                    self.state = Reactive(data, self.save)
                return True
        else:
            return False

    def save(self):
        if not os.path.exists(self.path):
            os.mkdir(self.path)
        with open(self.file, "w") as storage:
            storage.seek(0)
            storage.truncate()  # clear file
            data = self.state.read().replace("'", '"')
            data = json.loads(data)
            data = json.dumps(data, sort_keys=True, indent=4, separators=(',', ':'))
            storage.write(data)